% function L = curve_length (c, manifold)
%   Return the length of the curve 'c' measured according to the metric
%   represented by 'manifold'.
%
%   Input:
%     c:        DxN curve consisting of N points in D dimensions.
%     manifold: a manifold object.
%
%   Output:
%     L: a scalar representing the length of the curve c.
%
%   See also: create_manifold, curve_energy, compute_geodesic


%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function L = curve_length (c, manifold)
  [D, N] = size (c);
  Ms = metric_tensor (c, manifold);
  switch (lower (manifold.weight_scheme))
    case {'diagonal'}
      %% Integrate!
      d = 0.5 * (c (:, 2) - c (:, 1));
      L = sqrt (d.' * (Ms (:, 1) .* d));
      for n = 2:N-1
        d1 = 0.5 * (c (:, n) - c (:, n-1));
        d2 = 0.5 * (c (:, n+1) - c (:, n));
        L = L + sqrt (d1.' * (Ms (:, n) .* d1)) + sqrt (d2.' * (Ms (:, n) .* d2));
      end % for
      d = 0.5 * (c (:, N) - c (:, N-1));
      L = L +  sqrt (d.' * (Ms (:, N) .* d));
      
    otherwise
      %% Integrate!
      d = 0.5 * (c (:, 2) - c (:, 1));
      L = sqrt (d.' * Ms (:, :, 1) * d);
      for n = 2:N-1
        d1 = 0.5 * (c (:, n) - c (:, n-1));
        d2 = 0.5 * (c (:, n+1) - c (:, n));
        L = L + sqrt (d1.' * Ms (:, :, n) * d1) + sqrt (d2.' * Ms (:, :, n) * d2);
      end % for
      d = 0.5 * (c (:, N) - c (:, N-1));
      L = L +  sqrt (d.' * Ms (:, :, N) * d);
  end % switch
end % function
