% function [Ms, W, Wt] = metric_tensor (c, manifold)
%   Evaluate the metric tensor at specified points.

%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function [Ms, W, Wt] = metric_tensor (c, manifold)
  %% Get problem dimensions
  [D, N] = size (c);
  R = numel (manifold.mu);
  lambda = manifold.lambda;
  
  switch (manifold.weight_scheme)
    case 'diagonal'
      mu = [manifold.mu{:}]; % DxR
      M = [manifold.M{:}]; % DxR
      C = [manifold.C{:}]; % DxR
      d = repmat (mu(:), 1, N) - repmat (c, R, 1); % DRxN
      inner = reshape (d .* repmat (C(:), 1, N), [D, R, N]); % DxRxN
      Wt = reshape (exp (-0.5*lambda*sumsq (inner, 1)), [R, N]); % RxN
      %Wt (end, :) = 1; % XXX
      s = sum (Wt, 1);
      W = Wt ./ repmat (s, R, 1); % RxN
      Ms = M * W;

    case 'local gaussian'
      mu = manifold.mu;
      M = manifold.M;
      Ms = zeros (D^2, N);
      Wt = zeros (R, N);
      
      for r = 1:R
        d = c - repmat (mu {r}, 1, size (c, 2));
        C = manifold.C {r};
        Wt (r, :) = exp (-0.5 * lambda * sumsq (C * d));
      end % for

      %Wt (end, :) = 1; % XXX
      s = sum (Wt, 1);
      W = Wt ./ repmat (s, R, 1);
      for r = 1:R
        Ms = Ms + M{r} (:) * W (r, :);
      end % for
  
      Ms = reshape (Ms, [D, D, N]);
    
    case 'global gaussian'
      mu = manifold.mu;
      M = manifold.M;
      Ms = zeros (D^2, N);
      Wt = zeros (R, N);

      C = chol (manifold.Gamma);
      for r = 1:R
        d = c - repmat (mu {r}, 1, size (c, 2));
        Wt (r, :) = exp (-0.5 * lambda * sumsq (C * d));
      end % for
      
      %Wt (end, :) = 1; % XXX
      s = sum (Wt, 1);
      W = Wt ./ repmat (s, R, 1);
      for r = 1:R
        Ms = Ms + M{r} (:) * W (r, :);
      end % for
  
      Ms = reshape (Ms, [D, D, N]);

    case 'parzen'
      Xr = [manifold.mu{:}]; % DxN
      dist2 = distmat2 (Xr.', c.'); % RxN
      H = normpdf (sqrt (dist2), 0, sqrt (manifold.sigma2));
      sH = sum (H, 1); % 1xN
      s = manifold.rho + manifold.s_t * sH; % 1xN
      Ms = 1 ./ s;
      W = H;
    
    case 'local diagonal pca'
      S = NaN (D, N);
      Xr = [manifold.mu{:}]; % DxR
      W = NaN (N, R); % NxR
      for n = 1:N
        cn = c (:, n); % Dx1
        delta = Xr - repmat (cn, 1, R); % DxR
        dist2 = sumsq (delta, 1); % 1xR
        w_n = exp (-0.5 * dist2 / manifold.sigma2) / ((2*pi*manifold.sigma2)^(D/2)); % 1xR
        W (n, :) = w_n;
        S (:, n) = (delta.^2) * w_n.' + manifold.rho; % Dx1
      end % for
      Ms = 1./ S; % DxN
      Wt = S; % DxN
    
    case 'local pca'
      S = NaN (D^2, N);
      Xr = [manifold.mu{:}]; % DxR
      for n = 1:N
        cn = c (:, n); % Dx1
        delta = Xr - repmat (cn, 1, R); % DxR
        dist2 = sumsq (delta, 1); % 1xR
        w_n = exp (-0.5 * dist2 / manifold.sigma2) / ((2*pi*manifold.sigma2)^(D/2)); % 1xR
        %w_n = w_n / sum (w_n);
        %mu_n = Xr * w_n.';
        %delta = Xr - repmat (mu_n, 1, R); % DxR
        S (:, n) = 0;
        for r = 1:R
          tmp = delta (:, r) * delta (:, r).';
          S (:, n) = S (:, n) + w_n (r) * tmp (:);
        end % for
        tmp = inv (reshape (S (:, n), [D, D]) + manifold.rho * eye (D));
        S (:, n) = tmp (:);
        %S (:, n) = (delta.^2) * w_n.' + manifold.rho; % Dx1
      end % for
      Ms = reshape (S, [D, D, N]);
      %Ms = 1./ S;
      %% XXX: What else should we return?
    
    otherwise
      error ('metric_tensor: unknown weight scheme ''%s''', manifold.weight_scheme);
  end % switch

end % function

