% function [mu, sigma, logs] = ml_mean (P, manifold)
%   Compute the mean of a data set with respect to a smoothly changing metric.
%
%   Input:
%     P:        a NxD matrix of D-dimensional data.
%     manifold: a structure describing how the metric tensor changes. This is
%               created with the 'create_manifold' function.
%
%   Output:
%     mu:    the mean value represented as a D-vector.
%     sigma: the variance of the data; this is the quantity being minimized.
%     logs:  a NxD matrix containing the log maps of the data wrt. 'mu'.
%
%   See also:
%     compute_geodesic, create_manifold, ode45, myode45

%% Copyright 2012 Soren Hauberg
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <http://www.gnu.org/licenses/>.

function [mu, sigma, logs] = ml_mean (P, manifold, method, varargin)
  %% Default arguments
  if (nargin < 3)
    method = 'Pennec';
  end % if
  
  [num_points, D] = size (P);
  logs = [];

  switch (lower (method))
    case 'pennec'     
      %% Get possible options
      options = struct (varargin {:});
      if (~isfield (options, 'alpha'))
        options.alpha = 0.25;
      end % if
      if (~isfield (options, 'max_iter'))
        options.max_iter = 1;
      end % if
      
      %% Initial solution
      if (isfield (options, 'init'))
        mu = options.init;
      else
        mu = mean (P);
      end % if
      
      %% Initial step
      [mu, sigma, logs] = pennec_step (P, mu, manifold, options);
      for iter = 1:options.max_iter-1
        [new_mu, new_sigma, logs] = pennec_step (P, mu, manifold, options);
        %mu = 0.5 * (new_mu + mu); % XXX: Is this type of smoothing a good idea?
        mu = new_mu
        if (new_sigma < sigma)
          options.alpha = 1.1 * options.alpha;
        else
          options.alpha = 0.75 * options.alpha;
        end % if
        
        fprintf ('ml_mean: iter=%.3d\t sigma=%.2f\t new_sigma=%.2f\t alpha=%.2f\n', iter, sigma, new_sigma, options.alpha);
        
        if (abs (sigma - new_sigma) < 1e-3)
          break;
        end % if
        sigma = new_sigma;
      end % for
      
    case 'sturm'
      max_iter = 3;
      Idx = randperm (num_points);
      mu = P (Idx (1), :);
      for iter = 1:max_iter
        start_n = 1 + (iter==1);
        for n = start_n:num_points
          p = P (Idx (n), :);
          [c, l] = compute_geodesic (mu, p, manifold);
          lin = linspace (0, 1, size (c, 2));
          w = 1/(n + (iter-1)*num_points);

          for d = 1:D
            mu (d) = interp1 (lin, c (d, :), w, 'spline');
          end % for
        end % for
        Idx = randperm (num_points);
      end % for
    
      if (nargout > 1)
        sigma = 0;
        for n = 1:num_points
          [~, l] = compute_geodesic (mu, P (n, :), manifold);
          sigma = sigma + l^2;
        end % for
        sigma = sigma / num_points;
      end % if
    otherwise
      error ('ml_mean: unknown method for computing means ''%s''', method);
  end % switch
end % function

function [mu, sigma, logs] = pennec_step (P, mu, manifold, options)
  %% Map to tangent space
  [~, Lens, logs, fails] = compute_all_geodesics (P, mu, manifold);
  works = ~fails;
        
  %% Recompute mean
  mu_tangent = mean (logs (works, :));
  sigma = mean (Lens (works).^2);
  mu_curve = Exp (mu, options.alpha * mu_tangent, manifold);
  mu = mu_curve (:, end);
end % function

